package com.ningdatech.pmapi.dashboard.handle;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.core.util.CommonUtils;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.dashboard.constant.DashboardConstant;
import com.ningdatech.pmapi.dashboard.model.vo.ProtraitProjectStatusSituationVO;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProject;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectService;
import com.ningdatech.pmapi.projectlib.enumeration.InstTypeEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectStatusChange;
import com.ningdatech.pmapi.projectlib.service.INdProjectStatusChangeService;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.HistoryService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname ProjectStatusHandler
 * @Description
 * @Date 2023/10/9 14:11
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ProjectStatusHandler {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;

    private final INdProjectStatusChangeService statusChangeService;

    private final HistoryService historyService;
    
    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;
    
    /**
     * 不同的项目状态 去求不同的审批完成时间 审批类型的
     * @param project
     * @return
     */
    public ProtraitProjectStatusSituationVO generateApproveStatus(Project project,String statusName,Integer instType) {
        ProtraitProjectStatusSituationVO vo = new ProtraitProjectStatusSituationVO();
        if(Objects.isNull(project)){
            return vo;
        }
        vo.setStatusName(statusName);
        //查询 单位内部审核完成的时间
        String projectCode = project.getProjectCode();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, projectCode));
        List<Long> ids = projects.stream().map(Project::getId).collect(Collectors.toList());
        //查询 项目与实例关联表
        ProjectInst unitInst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, ids)
                .eq(ProjectInst::getInstType, instType)
                .last(BizConst.LIMIT_1));
        if(Objects.isNull(unitInst)){
            return vo;
        }
        //查询 这个实例 的完成时间
        HistoricProcessInstance instance = historyService.createHistoricProcessInstanceQuery()
                .finished()
                .processInstanceId(unitInst.getInstCode())
                .singleResult();
        if(Objects.isNull(instance)){
            return vo;
        }
        Date endTime = instance.getEndTime();
        vo.setStatusTime(BizUtils.convertDate(endTime));
        return vo;
    }

    /**
     * 不同的项目状态 去求不同的审批完成时间 非审批类型的
     * @param project
     * @return
     */
    public ProtraitProjectStatusSituationVO generateStatus(Project project,String statusName,
                                                           Integer beforeStatusCode,Integer afterStatusCode) {
        ProtraitProjectStatusSituationVO vo = new ProtraitProjectStatusSituationVO();
        if(Objects.isNull(project)){
            return vo;
        }
        vo.setStatusName(statusName);
        //查询 单位内部审核完成的时间
        String projectCode = project.getProjectCode();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectCode, projectCode));
        List<Long> ids = projects.stream().map(Project::getId).collect(Collectors.toList());
        //查询 状态机流转的时间点
        ProjectStatusChange statusChange = statusChangeService.getOne(Wrappers.lambdaQuery(ProjectStatusChange.class)
                .eq(ProjectStatusChange::getBeforeStatus, beforeStatusCode)
                .eq(ProjectStatusChange::getProjectCode, projectCode)
                .eq(ProjectStatusChange::getAfterStatus, afterStatusCode)
                .last(BizConst.LIMIT_1));

        if(Objects.isNull(statusChange)){
            return vo;
        }
        vo.setStatusTime(statusChange.getCreateOn());
        return vo;
    }

    /**
     * 绩效
     * @param project
     * @return
     */
    public ProtraitProjectStatusSituationVO generatePerformanceStatus(Project project) {
        ProtraitProjectStatusSituationVO vo = new ProtraitProjectStatusSituationVO();
        vo.setStatusName(DashboardConstant.Protrait.PERFORMANCE);
        if(Objects.isNull(project)){
            return vo;
        }
        //查询 绩效完成 核查时间
        String projectCode = project.getProjectCode();
        PerformanceAppraisalProject performance = performanceAppraisalProjectService.getOne(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getProjectCode,projectCode)
                .isNotNull(PerformanceAppraisalProject::getVerifyTotalScore)
                .isNotNull(PerformanceAppraisalProject::getVerifyScoreTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(performance)){
            vo.setStatusTime(performance.getVerifyScoreTime());
        }
        return vo;
    }
}
