package com.ningdatech.pmapi.workbench.manage;

import com.google.common.collect.Lists;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.projectdeclared.manage.DefaultDeclaredProjectManage;
import com.ningdatech.pmapi.projectlib.manage.ProjectLibManage;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.sys.manage.NoticeManage;
import com.ningdatech.pmapi.sys.model.req.NoticeListReq;
import com.ningdatech.pmapi.sys.model.req.WarningListReq;
import com.ningdatech.pmapi.sys.model.vo.WflowEarlyWarningRecordsVO;
import com.ningdatech.pmapi.sys.service.IEarlyWarningRecordsService;
import com.ningdatech.pmapi.todocenter.manage.TodoCenterManage;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedReq;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.ningdatech.pmapi.workbench.converter.WorkbenchConverter;
import com.ningdatech.pmapi.workbench.model.vo.WorkbenchVO;
import com.wflow.enums.WarningRuleTypeEnum;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ForkJoinPool;
import java.util.stream.Collectors;

/**
 * @Classname WorkbenchManage
 * @Description
 * @Date 2023/3/20 10:56
 * @Author PoffyZhang
 */
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;

    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;

    private final IEarlyWarningRecordsService earlyWarningRecordsService;

    private final NoticeManage noticeManage;

    private final UserInfoHelper userInfoHelper;

    private final ProjectLibManage projectLibManage;

    public WorkbenchVO getWorkbenchData(Integer year) {
        WorkbenchVO res = new WorkbenchVO();
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);

        CompletableFuture.allOf(
                CompletableFuture.runAsync(() -> {
                    //1.待办中心数据
                    TodoCenterStatisticsVO statisticsVO = todoCenterManage.todoCenterStatistics(user.getEmployeeCode());
                    ToBeProcessedReq toBeProcessedReq = new ToBeProcessedReq();
                    toBeProcessedReq.setPageNumber(1);
                    toBeProcessedReq.setPageSize(5);
                    toBeProcessedReq.setLoginUserId(userId);
                    toBeProcessedReq.setEmployeeCode(user.getEmployeeCode());
                    statisticsVO.setTodoList(todoCenterManage.todoProjectList(toBeProcessedReq).getRecords()
                            .stream().map(WorkbenchConverter::convert).collect(Collectors.toList()));
                    res.setTodoCerter(statisticsVO);
                }, ForkJoinPool.commonPool()),
                CompletableFuture.runAsync(() -> {
                    //2.项目统计数据
                    res.setOrgDeclared(WorkbenchConverter.convert(defaultDeclaredProjectManage
                            .declaredProjectOrgStatistics(year, user),defaultDeclaredProjectManage.declaredProjectOrgStatistics(year - 1, user)));
                    if (userInfoHelper.isSuperOrRegionAdmin(user.getUserId())) {
                        res.setRegionDeclared(WorkbenchConverter.convert(defaultDeclaredProjectManage
                                .declaredProjectRegionStatistics(year, user),defaultDeclaredProjectManage.declaredProjectRegionStatistics(year - 1, user)));
                    }
                }, ForkJoinPool.commonPool()),
                CompletableFuture.runAsync(() -> {
                    //3.所有公告按类型分
                    NoticeListReq noticeListReq = new NoticeListReq();
                    noticeListReq.setPageNumber(1);
                    noticeListReq.setPageSize(1000);
                    res.setNoticeList(noticeManage.listToMapByManager(noticeListReq,user));
                }, ForkJoinPool.commonPool()),
                CompletableFuture.runAsync(() -> {
                    //4.预警记录
                    List<WorkbenchVO.EarlyWarning> eaylies = Lists.newArrayList();
                    WarningListReq req = new WarningListReq();
//                    req.setBuildOrgCode(user.getEmpPosUnitCode());
                    WorkbenchVO.EarlyWarning early1 = new WorkbenchVO.EarlyWarning();
                    early1.setType(WarningRuleTypeEnum.PROCESS_WARNING.getCode());
                    early1.setTypeName(WarningRuleTypeEnum.PROCESS_WARNING.getDesc());
                    PageVo<WflowEarlyWarningRecordsVO> records1 = earlyWarningRecordsService
                            .records(WarningRuleTypeEnum.PROCESS_WARNING.getCode(), req);
                    early1.setTotal(records1.getTotal().intValue());
                    early1.setRecords(Lists.newArrayList(records1.getRecords()));
                    eaylies.add(early1);
                    WorkbenchVO.EarlyWarning early2 = new WorkbenchVO.EarlyWarning();
                    early2.setType(WarningRuleTypeEnum.DECLARED_WARNING.getCode());
                    early1.setTypeName(WarningRuleTypeEnum.DECLARED_WARNING.getDesc());
                    PageVo<WflowEarlyWarningRecordsVO> records2 = earlyWarningRecordsService
                            .records(WarningRuleTypeEnum.DECLARED_WARNING.getCode(), req);
                    early2.setRecords(Lists.newArrayList(records2.getRecords()));
                    early2.setTotal(records2.getTotal().intValue());
                    eaylies.add(early2);
                    WorkbenchVO.EarlyWarning early3 = new WorkbenchVO.EarlyWarning();
                    early3.setType(WarningRuleTypeEnum.OPERATION_WARNING.getCode());
                    early3.setTypeName(WarningRuleTypeEnum.OPERATION_WARNING.getDesc());
                    PageVo<WflowEarlyWarningRecordsVO> records3 = earlyWarningRecordsService
                            .records(WarningRuleTypeEnum.OPERATION_WARNING.getCode(), req);
                    early3.setRecords(Lists.newArrayList(records3.getRecords()));
                    early3.setTotal(records3.getTotal().intValue());
                    eaylies.add(early3);
                    WorkbenchVO.EarlyWarning early4 = new WorkbenchVO.EarlyWarning();
                    early4.setType(WarningRuleTypeEnum.RENEWAL_FUND.getCode());
                    early4.setTypeName(WarningRuleTypeEnum.RENEWAL_FUND.getDesc());
                    PageVo<WflowEarlyWarningRecordsVO> records4 = earlyWarningRecordsService
                            .records(WarningRuleTypeEnum.RENEWAL_FUND.getCode(), req);
                    early4.setRecords(Lists.newArrayList(records4.getRecords()));
                    early4.setTotal(records4.getTotal().intValue());
                    eaylies.add(early4);
                    res.setEarlyWarning(eaylies);
                }, ForkJoinPool.commonPool())
        ).join();

        //这个不能放入子线程
        ProjectListReq projectListReq = new ProjectListReq();
        projectListReq.setPageNumber(1);
        projectListReq.setPageSize(5);
        projectListReq.setProjectYear(year);
        projectListReq.setBuildOrgCode(user.getEmpPosUnitCode());
        res.setProjects(new ArrayList<>(projectLibManage.projectLibListApprove(projectListReq, user).getRecords()));
        return res;
    }
}
