package com.ningdatech.pmapi.workbench.model.vo;

import com.ningdatech.pmapi.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.pmapi.sys.model.vo.NoticeListItemVO;
import com.ningdatech.pmapi.sys.model.vo.WflowEarlyWarningRecordsVO;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Classname WorkbenchVO
 * @Description
 * @Date 2023/3/20 9:33
 * @Author PoffyZhang
 */
@Data
public class WorkbenchVO implements Serializable {

    @ApiModelProperty("待办中心统计数据 以及代办处理")
    public TodoCenterStatisticsVO todoCerter;

    @ApiModelProperty("单位申报管理统计数据")
    public DeclaredStatistics orgDeclared;

    @ApiModelProperty("区域申报管理统计数据 只有区域管理员才有数据")
    public DeclaredStatistics regionDeclared;

    @ApiModelProperty("项目动态")
    public List<ProjectLibListItemVO> projects;

    @ApiModelProperty("公告列表")
    public Map<String,List<NoticeListItemVO>> noticeList;

    @ApiModelProperty("预警记录")
    public List<EarlyWarning> earlyWarning;

    @Data
    public static class EarlyWarning {
        @ApiModelProperty("类型")
        private Integer type;

        @ApiModelProperty("类型名称")
        private String typeName;

        @ApiModelProperty("总数")
        private Integer total;

        @ApiModelProperty("预警记录")
        private List<WflowEarlyWarningRecordsVO> records;
    }

    @Data
    public static class DeclaredStatistics {
        @ApiModelProperty("总项目数")
        private Integer totalNum;
        @ApiModelProperty("建设项目数")
        private Integer buildNum;
        @ApiModelProperty("运维项目数")
        private Integer operationNum;

        @ApiModelProperty("待立项数")
        private Integer notApprovedNum;
        @ApiModelProperty("已立项数")
        private Integer approvedNum;
        @ApiModelProperty("建设中数")
        private Integer constructionNum;
        @ApiModelProperty("待验收数")
        private Integer tobeInspectedNum;
        @ApiModelProperty("已归档数")
        private Integer archivedNum;

        @ApiModelProperty("申报总金额")
        private BigDecimal declaredAmount = BigDecimal.ZERO;
        @ApiModelProperty("立项总金额")
        private BigDecimal approvalAmount = BigDecimal.ZERO;


        @ApiModelProperty("申报总金额-去年")
        private BigDecimal declaredAmountLastYear = BigDecimal.ZERO;
        @ApiModelProperty("立项总金额-去年")
        private BigDecimal approvalAmountLastYear = BigDecimal.ZERO;
        @ApiModelProperty("申报总金额-较去年 增长")
        private BigDecimal declaredAmountGrowthRate;
        @ApiModelProperty("立项总金额-较去年 增长")
        private BigDecimal approvalAmountGrowthRate;

        public BigDecimal getDeclaredAmountGrowthRate(){
            if(Objects.isNull(declaredAmount)){
                this.declaredAmount = BigDecimal.ZERO;
            }

//            if((Objects.nonNull(declaredAmount)&&declaredAmount.compareTo(BigDecimal.ZERO) > 0) && (Objects.isNull(declaredAmountLastYear) ||
//                    declaredAmountLastYear.compareTo(BigDecimal.ZERO) == 0)){
//                return BigDecimal.valueOf(100);
//            }

            if(Objects.nonNull(declaredAmountLastYear) && declaredAmountLastYear.compareTo(BigDecimal.ZERO) > 0){
                return declaredAmount.subtract(declaredAmountLastYear).multiply(BigDecimal.valueOf(100))
                        .divide(declaredAmountLastYear,BigDecimal.ROUND_CEILING)
                        .setScale(BigDecimal.ROUND_CEILING, RoundingMode.CEILING);
            }
            return BigDecimal.ZERO;
        }

        public BigDecimal getApprovalAmountGrowthRate(){
            if(Objects.isNull(approvalAmount)){
                this.approvalAmount = BigDecimal.ZERO;
            }
//            if((Objects.nonNull(approvalAmount)&&approvalAmount.compareTo(BigDecimal.ZERO) > 0) && (Objects.isNull(approvalAmountLastYear) ||
//                    approvalAmountLastYear.compareTo(BigDecimal.ZERO) == 0)){
//                return BigDecimal.valueOf(100);
//            }

            if(Objects.nonNull(declaredAmountLastYear) && approvalAmountLastYear.compareTo(BigDecimal.ZERO) > 0){
                return approvalAmount.subtract(approvalAmountLastYear).multiply(BigDecimal.valueOf(100))
                        .divide(approvalAmountLastYear,BigDecimal.ROUND_CEILING)
                        .setScale(BigDecimal.ROUND_CEILING, RoundingMode.CEILING);
            }
            return BigDecimal.ZERO;
        }
    }
}
