package com.ningdatech.pmapi.projectCollection;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.log.model.domain.OptLog;
import com.ningdatech.log.service.OptLogService;
import com.ningdatech.pmapi.AppTests;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectApplyDTO;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectBaseinfoDTO;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectSaveDTO;
import com.ningdatech.pmapi.gov.model.entity.*;
import com.ningdatech.pmapi.gov.service.*;
import com.ningdatech.pmapi.projectdeclared.utils.GenerateProjectCodeUtil;
import com.ningdatech.pmapi.scheduler.task.SynProjectCollectionTask;
import com.ningdatech.pmapi.scheduler.task.SynProjectCoreBizTask;
import org.apache.commons.lang3.StringUtils;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.net.UnknownHostException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Classname ProjectCollectionTest
 * @Description
 * @Date 2023/9/7 15:08
 * @Author PoffyZhang
 */
public class ProjectCollectionTest extends AppTests {

    @Autowired
    private IGovBizProjectBaseinfoService baseinfoService;

    @Autowired
    private IGovBizProjectApplyService applyService;

    @Autowired
    private IGovBizProjectApproveService approveService;

    @Autowired
    private IGovBizProjectCimplementService cimplementService;

    @Autowired
    private IGovBizProjectMimplementService mimplementService;

    @Autowired
    private IGovBizProjectProcureService procureService;

    @Autowired
    private GenerateProjectCodeUtil generateProjectCodeUtil;

    @Autowired
    private SynProjectCollectionTask collectionTask;

    @Autowired
    private OptLogService optLogService;

    @Autowired
    private SynProjectCoreBizTask projectCoreBizTask;

    @Test
    public void test(){
        List<String> baseProjIds = Lists.newArrayList("33118100020200100002");
        for(String baseProjId : baseProjIds){
            GovBizProjectBaseinfo baseinfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                    .eq(GovBizProjectBaseinfo::getBaseProjId, baseProjId)
                    .orderByDesc(GovBizProjectBaseinfo::getTongTime)
                    .last(BizConst.LIMIT_1));
            GovBizProjectApply apply = applyService.getOne(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .eq(GovBizProjectApply::getBaseProjId, baseProjId)
                    .orderByDesc(GovBizProjectApply::getTongTime)
                    .last(BizConst.LIMIT_1));
            if(Objects.isNull(baseinfo)){
                System.out.println("项目不存在 直接下一个");
                continue;
            }
            GovBizProjectSaveDTO saveDTO = new GovBizProjectSaveDTO();
            saveDTO.setApply(BeanUtil.copyProperties(apply, GovBizProjectApplyDTO.class));
            saveDTO.setBaseinfo(BeanUtil.copyProperties(baseinfo, GovBizProjectBaseinfoDTO.class));
            String newProjId = generateProjectCodeUtil.generateProjectCode(saveDTO);

            baseinfo.setBaseProjId(newProjId);
            baseinfoService.updateById(baseinfo);
            apply.setBaseProjId(newProjId);
            applyService.updateById(apply);

            GovBizProjectApprove approve = approveService.getOne(Wrappers.lambdaUpdate(GovBizProjectApprove.class)
                    .eq(GovBizProjectApprove::getBaseProjId, baseProjId)
                    .orderByDesc(GovBizProjectApprove::getTongTime)
                    .last(BizConst.LIMIT_1));
            if(Objects.nonNull(approve)){
                approve.setBaseProjId(newProjId);
                approveService.updateById(approve);
            }


            GovBizProjectCimplement cimplement = cimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                    .eq(GovBizProjectCimplement::getBaseProjId, baseProjId)
                    .orderByDesc(GovBizProjectCimplement::getTongTime)
                    .last(BizConst.LIMIT_1));
            if(Objects.nonNull(cimplement)){
                cimplement.setBaseProjId(newProjId);
                cimplement.setBaseBidCode(newProjId);
                cimplementService.updateById(cimplement);
            }

            GovBizProjectMimplement mimplement = mimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                    .eq(GovBizProjectMimplement::getBaseProjId, baseProjId)
                    .orderByDesc(GovBizProjectMimplement::getTongTime)
                    .last(BizConst.LIMIT_1));
            if(Objects.nonNull(mimplement)){
                mimplement.setBaseProjId(newProjId);
                mimplement.setBaseBidCode(newProjId);
                mimplementService.updateById(mimplement);
            }

            List<GovBizProjectProcure> procures = procureService.list(Wrappers.lambdaUpdate(GovBizProjectProcure.class)
                    .eq(GovBizProjectProcure::getBaseProjId, baseProjId));

            for(GovBizProjectProcure procure : procures){
                procure.setBaseProjId(newProjId);
                procure.setBaseBidCode(newProjId + "-" + procure.getBaseBidCode().split("-")[1]);
                procureService.updateById(procure);
            }
        }
    }

    @Test
    public void test2() throws UnknownHostException {
        collectionTask.doTask();
    }

    @Test
    public void test3() throws UnknownHostException {
        collectionTask.stagingConvertPdf("33112200020230100008");
    }

    /**
     * 去补文件
     */
    @Test
    public void test4(){
        List<GovBizProjectApprove> approves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                .eq(GovBizProjectApprove::getApprovalFile,"null"));

        System.out.println("approves size :" + approves.size());

        for(GovBizProjectApprove approve : approves){
            //https://sjxmglxt-oss.oss-cn-hangzhou-zjzwy01-d01-a.cloud-inner.zj.gov.cn/20230912/6a5c00def2db41b199d8cc99ff04728e.pdf
            String baseProjId = approve.getBaseProjId();

            List<OptLog> opLogs = optLogService.list(Wrappers.lambdaQuery(OptLog.class)
                    .like(OptLog::getParams, approve.getBaseProjName())
                    .like(OptLog::getParams, "\"approvalFile\"")
                    .last(BizConst.LIMIT_1));
            if(CollUtil.isEmpty(opLogs)){
                System.out.println("没有这个记录");
                continue;
            }

            for(OptLog opLog : opLogs){
                System.out.println(opLog.getParams());

                if(StringUtils.isNotBlank(opLog.getParams())){
                    StringJoiner sj = new StringJoiner("；");
                    JSONArray jsonArray = JSON.parseArray(opLog.getParams());
                    jsonArray.forEach(j -> {
                        JSONObject json = JSON.parseObject(JSON.toJSONString(j));
                        JSONObject approveJson = json.getJSONObject("approve");
                        if(Objects.nonNull(approveJson)){
                            String approvalFile = approveJson.getString("approvalFile");
                            System.out.println("approvalFile:" + approvalFile);
                            if(StringUtils.isBlank(approvalFile) || "null".equals(approvalFile)){
                                return;
                            }
                            sj.add(approvalFile);
                        }
                    });
                    if(StringUtils.isNotBlank(sj.toString())){
                        approve.setApprovalFile(sj.toString());
                        approveService.updateById(approve);
                        break;
                    }
                }
            }
        }
    }

    @Test
    public void test5(){
        List<GovBizProjectApply> list = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .isNotNull(GovBizProjectApply::getBaseProjStartTime)
                .isNotNull(GovBizProjectApply::getBaseProjEndTime)
                .isNull(GovBizProjectApply::getBaseProjDuration));

        for(GovBizProjectApply apply : list){
            apply.setBaseProjDuration(convertMonth(apply.getBaseProjStartTime(),apply.getBaseProjEndTime()));
            applyService.updateById(apply);
        }
    }

    public static String convertMonth(LocalDateTime baseProjStartTime, LocalDateTime baseProjEndTime) {
        Temporal temporal1 = baseProjStartTime.toLocalDate();
        Temporal temporal2 = baseProjEndTime.toLocalDate();
        // 方法返回为相差月份
        Long ll = ChronoUnit.MONTHS.between(temporal1, temporal2);
        return ll.toString();
    }

    @Test
    public void test6(){
        List<GovBizProjectProcure> list = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                .eq(GovBizProjectProcure::getDeleted,Boolean.FALSE));

        for(GovBizProjectProcure procure : list){
            try{
                String index = procure.getBaseBidCode().split("-")[1];
                procure.setBaseBidCode(procure.getBaseProjId() + StrPool.DASH + String.format(StrPool.FORMAT_DATE_SINGLE_DIGITS,Integer.parseInt(index)));
                procureService.updateById(procure);
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    @Test
    public void test7() throws UnknownHostException {
        String orgCode = "GO_5f1f6f28a6f7484ea3b9991d8d02599f";
        projectCoreBizTask.doTask(orgCode);
    }
}
